/*
 * MIT License
 *
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package io.manbang.wireless.thresh;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.PixelFormat;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.SystemClock;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.view.accessibility.AccessibilityNodeProvider;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;

import io.manbang.wireless.thresh.channel.ThreshBridgeApapter;
import io.manbang.wireless.thresh.channel.ThreshMethodChannel;
import io.manbang.wireless.thresh.jsruntime.JSEngine;
import io.manbang.wireless.thresh.load.BundleCallback;
import io.manbang.wireless.thresh.load.BundleLoader;
import io.manbang.wireless.thresh.load.BundleManager;
import io.manbang.wireless.thresh.util.CompactUtil;
import io.manbang.wireless.thresh.util.ThreshLogger;
import io.manbang.wireless.thresh.util.EnumHostType;
import io.manbang.wireless.thresh.view.ThreshFlutter;
import io.manbang.wireless.thresh.view.platform.ViewRegistrant;

import java.util.Map;

import io.flutter.BuildConfig;
import io.flutter.plugins.GeneratedPluginRegistrant;
import io.flutter.view.AccessibilityBridge;
import io.flutter.view.FlutterMain;
import io.flutter.view.FlutterRunArguments;
import io.flutter.view.FlutterView;

/**
 * Thresh承载页面，测试模式使用不到
 */
public class ThreshActivity extends Activity {

    private static final String TAG = "ThreshActivity";

    private static final String EXTRA_ROUTER = "router";

    private int screenWidth;

    public String mPageName;
    /**
     * 业务名称
     */
    public String mBundleName;

    private LinearLayout rootView;

    private View placeHolder;

    private FlutterView flutterView;

    private View errorView;

    private TextView tvError;

    private FrameLayout.LayoutParams rootViewLp;

    public JSEngine mDynamicJSEngine;

    private ThreshBridgeApapter dynamicFBridgeAdapter;

    private ThreshMethodChannel threshMethodChannel;

    private String mRouter;

    private long startRenderTime;

    private boolean isReported = false;

    private boolean isStopped = false;

    ViewRegistrant mPluginReistrant;

    public static Intent buildIntent(Context context, String router) {
        Intent intent = new Intent(context, ThreshActivity.class);
        intent.putExtra(EXTRA_ROUTER, router);
        return intent;
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        startRenderTime = SystemClock.elapsedRealtime();
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_thresh_container);
        setStatusBarFullTransparent();
        screenWidth = getResources().getDisplayMetrics().widthPixels;
        mRouter = getIntent().getStringExtra(EXTRA_ROUTER);
        Uri uri = Uri.parse("ymm://flutter." + mRouter);
        mBundleName = uri.getQueryParameter("biz");
        mPageName = uri.getQueryParameter("page");
        mRouter = Uri.parse(mRouter).buildUpon().appendQueryParameter("buildType", BuildConfig.DEBUG ? "0" : "2").build().toString();
        if (TextUtils.isEmpty(mBundleName) || TextUtils.isEmpty(mPageName)) {
            finish();
            return;
        }

        rootViewLp = new FrameLayout.LayoutParams(screenWidth * 2,
                                                  LinearLayout.LayoutParams.MATCH_PARENT);
        final LinearLayout.LayoutParams placeHolderLp = new LinearLayout.LayoutParams(screenWidth,
                                                                                      LinearLayout.LayoutParams.MATCH_PARENT);
        rootView = (LinearLayout) findViewById(R.id.contentView);

        rootView.setLayoutParams(rootViewLp);

        placeHolder = findViewById(R.id.place_holder);

        placeHolder.setLayoutParams(placeHolderLp);

        errorView = findViewById(R.id.ly_error);
        tvError = findViewById(R.id.tv_error);

        errorView.setVisibility(View.GONE);
        View ivLeft = findViewById(R.id.iv_left);
        ivLeft.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
        BundleManager.getBundlePath(this, mBundleName, new BundleManager.BundleListener() {
            @Override
            public void onResult(boolean isDebugMode, String uuid, String bundlePath) {
                initFlutter(isDebugMode, bundlePath);
            }
        });
    }

    private void initFlutter(boolean isDebugMode,String bundlePath){
        mRouter = Uri.parse(mRouter).buildUpon().appendQueryParameter("jsBundlePath", bundlePath).build().toString();
        flutterView = ThreshFlutter.createView(this, mRouter);
        flutterView.getHolder().setFormat(PixelFormat.TRANSLUCENT);
        flutterView.addFirstFrameListener(new FlutterView.FirstFrameListener() {
            @Override
            public void onFirstFrame() {
                placeHolder.setVisibility(View.GONE);
                rootViewLp.width = screenWidth;
                rootView.setLayoutParams(rootViewLp);

                if (isReported || isStopped) {
                    return;
                }

                final long renderTime = SystemClock.elapsedRealtime() - startRenderTime;
                isReported = true;
            }
        });
        rootView.addView(flutterView, new LinearLayout.LayoutParams(screenWidth,
                                                                    LinearLayout.LayoutParams.MATCH_PARENT));

        flutterViewOnCreate();

        dynamicFBridgeAdapter = new ThreshBridgeApapter();

        mDynamicJSEngine = new JSEngine(mBundleName, this, dynamicFBridgeAdapter);
        mDynamicJSEngine.getJsExecutor().withHostType(EnumHostType.APP_HOST);

        threshMethodChannel = new ThreshMethodChannel(this, flutterView, mDynamicJSEngine, dynamicFBridgeAdapter);
        threshMethodChannel.register(mDynamicJSEngine);

        if (isDebugMode){
            BundleLoader.loadBundleJsDebug(mDynamicJSEngine, "127.0.0.1:12345",
                                           new BundleCallback() {
                                               @Override
                                               public void onResult(int code, String reason) {
                                                   if (code == 0) {
                                                       errorView.setVisibility(View.GONE);
                                                       rootView.setVisibility(View.VISIBLE);
                                                   } else {
                                                       FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(
                                                               ViewGroup.LayoutParams.MATCH_PARENT,
                                                               ViewGroup.LayoutParams.MATCH_PARENT);
                                                       params.setMargins(0, CompactUtil
                                                                                 .getStatusBarHeight(
                                                                                         ThreshActivity.this),
                                                                         0, 0);
                                                       errorView.setLayoutParams(params);
                                                       errorView.setVisibility(View.VISIBLE);
                        rootView.setVisibility(View.GONE);
                        tvError.setText(reason);
                    }
                }
            });
        }else {
            if (TextUtils.isEmpty(bundlePath)){
                FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
                params.setMargins(0, CompactUtil.getStatusBarHeight(ThreshActivity.this), 0, 0);
                errorView.setLayoutParams(params);
                errorView.setVisibility(View.VISIBLE);
                rootView.setVisibility(View.GONE);
            }else {
                errorView.setVisibility(View.GONE);
                rootView.setVisibility(View.VISIBLE);
                mDynamicJSEngine.bundlePath = bundlePath;
                BundleLoader.loadBundleJs(mDynamicJSEngine, mDynamicJSEngine.bundlePath,
                                          mBundleName, mPageName);
            }
        }
    }

    private void flutterViewOnCreate() {
        if (flutterView != null) {
            final FlutterRunArguments arguments = new FlutterRunArguments();
            arguments.bundlePath = FlutterMain.findAppBundlePath();
            arguments.entrypoint = "main";
            flutterView.runFromBundle(arguments);
            GeneratedPluginRegistrant.registerWith(flutterView.getPluginRegistry());
            mPluginReistrant = new ViewRegistrant();
            mPluginReistrant.registerWith(this,flutterView.getPluginRegistry(),mBundleName,mPageName);
        }
    }

    /**
     * 更新nativeView能力
     */
    public void updateNativeView(Map params){
        if (mPluginReistrant != null){
            mPluginReistrant.refreshView(params);
        }
    }

    protected void setStatusBarFullTransparent() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Window window = getWindow();
            window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
            window.getDecorView().setSystemUiVisibility(
                    View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN | View.SYSTEM_UI_FLAG_LAYOUT_STABLE);
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
            window.setStatusBarColor(Color.TRANSPARENT);
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        }
    }
    @Override
    protected void onStart() {
        super.onStart();
        if (flutterView != null) {
            flutterView.onStart();
        }
        ThreshLogger.i("thresh init_flutter", "start");
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (flutterView != null) {
            flutterView.onPostResume();
        }
        if(mDynamicJSEngine != null){
            mDynamicJSEngine.pageOnShow();
        }
        if (mPluginReistrant != null){
            mPluginReistrant.onResume();
        }
        ThreshLogger.i("thresh init_flutter", "show");
    }

    @Override
    protected void onPause() {
        super.onPause();
        ThreshLogger.i("thresh init_flutter", "hide start");
        if (flutterView != null) {
            flutterView.onPause();
        }
        if(mDynamicJSEngine != null){
            mDynamicJSEngine.pageOnHide();
        }
        if (mPluginReistrant != null){
            mPluginReistrant.onPause();
        }
        ThreshLogger.i("thresh init_flutter", "hide end");
    }

    @Override
    protected void onStop() {
        super.onStop();
        ThreshLogger.i("thresh init_flutter", "stop start");
        if (flutterView != null) {
            flutterView.onStop();
        }

        isStopped = true;
        ThreshLogger.i("thresh init_flutter", "stop end");
    }

    @Override
    protected void onDestroy() {
        try {
            ThreshLogger.i("thresh init_flutter", "destroy start");
            if (mDynamicJSEngine != null){
                mDynamicJSEngine.onDestory();
            }
            if (dynamicFBridgeAdapter != null) {
                dynamicFBridgeAdapter.unRegister();
            }
            if (threshMethodChannel != null) {
                threshMethodChannel.unRegister();
            }
            if (flutterView != null) {
                try {
                    AccessibilityNodeProvider accessibilityNodeProvider = flutterView.getAccessibilityNodeProvider();
                    if (accessibilityNodeProvider != null && accessibilityNodeProvider instanceof AccessibilityBridge) {
                        ((AccessibilityBridge) accessibilityNodeProvider).release();
                    }
                } catch (Throwable e) {
                    e.printStackTrace();
                }
                flutterView.destroy();
            }
            if (mDynamicJSEngine != null){
                mDynamicJSEngine.close();
            }
            if (mPluginReistrant != null){
                mPluginReistrant.onDestroy();
            }
            ThreshLogger.i("thresh init_flutter", "destroy end");
        } catch (Exception e) {
            e.printStackTrace();
            ThreshLogger.i("thresh init_flutter", "destory error" + e.getMessage());
        }
        super.onDestroy();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public void onBackPressed() {
        if(mDynamicJSEngine != null){
            mDynamicJSEngine.nativePop();
        }
    }

}
