/*
 * MIT License
 *
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package io.manbang.wireless.thresh.load;

import android.text.TextUtils;

import io.manbang.wireless.thresh.jsruntime.JSEngine;
import io.manbang.wireless.thresh.util.ThreshLogger;
import io.manbang.wireless.thresh.util.FileUtils;

import java.io.IOException;

import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.http.GET;

/**
 * bundle js 加载
 */
public class BundleLoader {

    static final String BUNDLE_URL = "127.0.0.1:12345";

    /**
     * 
     */
    public static void loadBundleJs(JSEngine jsEngine, String path, String bundleName,
                                    String page) {
        ThreshLogger.i("getFromFile start");
        String script = FileUtils.getFromFile(path + "/bundle.js");
        boolean isSuccess = !TextUtils.isEmpty(script);
        if (isSuccess) {
            ThreshLogger.i("executeScript File start");
            jsEngine.getJsExecutor().executeScript(script, null);
            ThreshLogger.i("executeScript File end");
        }
    }

    public static void loadBundleJsDebug(JSEngine jsEngine, String localIP,
                                         BundleCallback callback) {
        //构建Retrofit
        Retrofit retrofit = new Retrofit.Builder().baseUrl("http://" + localIP + "/").build();
        BundleDebugMode bundleData = retrofit.create(BundleDebugMode.class);
        //调用接口方法并返回
        final Call<ResponseBody> data = bundleData.getData();
        //网络请求
        data.enqueue(new Callback<ResponseBody>() {
            @Override
            public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {
                try {
                    if (response != null && response.body() != null) {
                        String script = response.body().source().readUtf8();
                        if (null == script) {
                            script = "";
                        }
                        jsEngine.getJsExecutor().executeScript(script, null);
                        if (callback != null){
                            callback.onResult(0,"加载JS成功");
                        }
                    } else {
                        if (callback != null){
                            callback.onResult(-1,"服务器未启动或JS异常");
                        }
                    }


                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onFailure(Call<ResponseBody> call, Throwable t) {
                if (callback != null) {
                    callback.onResult(-1, "服务器未启动或JS异常");
                }
            }
        });
    }

    public static void loadBundleJsDebug(JSEngine jsEngine, BundleCallback callback) {
        //构建Retrofit
        loadBundleJsDebug(jsEngine, BUNDLE_URL, callback);
    }

    interface BundleDebugMode {

        @GET("bundle.js")
        Call<ResponseBody> getData();
    }
}
