/*
 * MIT License
 * <p>
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * <p>
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * <p>
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package io.manbang.wireless.thresh.view;

import android.app.Activity;
import android.arch.lifecycle.Lifecycle;
import android.content.Context;
import android.content.Intent;
import android.support.annotation.NonNull;

import io.manbang.wireless.thresh.ThreshActivity;
import io.manbang.wireless.thresh.util.ThreshLogger;

import io.flutter.plugin.common.BasicMessageChannel;
import io.flutter.plugin.common.StringCodec;
import io.flutter.view.FlutterMain;
import io.flutter.view.FlutterNativeView;
import io.flutter.view.FlutterView;

/**
 *  Main entry point for using Flutter in Android applications.
 */
public final class ThreshFlutter {

    private ThreshFlutter() {
        // to prevent instantiation
    }

    /**
     * Initiates the Dart VM. Calling this method at an early point may help decreasing time to first
     * frame for a subsequently created {@link FlutterView}.
     *
     * @param applicationContext the application's {@link Context}
     */
    public static void startInitialization(@NonNull Context applicationContext) {
        FlutterMain.startInitialization(applicationContext);
    }

    /**
     * Build an intent for a flutter activity.
     * @param context
     * @param router
     */
    public static Intent buildIntent(Context context, String router) {
        ThreshLogger.d("buildIntent = " + context);
        return ThreshActivity.buildIntent(context, router);
    }

    /**
     * Creates a {@link FlutterView} linked to the specified {@link Activity} and {@link Lifecycle}.
     * The optional initial route string will be made available to the Dart code (via
     * {@code window.defaultRouteName}) and may be used to determine which widget should be displayed
     * in the view. The default initialRoute is "/".
     *
     * @param activity an {@link Activity}
     * @param initialRoute an initial route {@link String}, or null
     * @return a {@link FlutterView}
     */
    @NonNull
    public static FlutterView createView(@NonNull final Activity activity,
                                         final String initialRoute) {
        FlutterMain.startInitialization(activity.getApplicationContext());
        FlutterMain.ensureInitializationComplete(activity.getApplicationContext(), null);
        final FlutterNativeView nativeView = new FlutterNativeView(activity);
        final FlutterView flutterView = new ThreshFlutterView(activity, null, nativeView) {
            private final BasicMessageChannel<String> lifecycleMessages = new BasicMessageChannel<>(
                    this, "flutter/lifecycle", StringCodec.INSTANCE);

            @Override
            public void onFirstFrame() {
                super.onFirstFrame();
                setAlpha(1.0f);
            }

            @Override
            public void onPostResume() {
                // Overriding default behavior to avoid dictating system UI via PlatformPlugin.
                lifecycleMessages.send("AppLifecycleState.resumed");
            }
        };
        if (initialRoute != null) {
            flutterView.setInitialRoute(initialRoute);
        }
        flutterView.setAlpha(0.0f);
        return flutterView;
    }
}
